@@ -690,6 +690,11 @@ static int vdi_snapshot(int argc, char **argv)
 	if (ret != EXIT_SUCCESS)
 		return ret;
 
+	if (inode->store_policy) {
+		sd_err("creating a snapshot of hypervolume is not supported");
+		return EXIT_FAILURE;
+	}
+
 	ret = dog_write_object(vid_to_vdi_oid(vid), 0,
 			       vdi_cmd_data.snapshot_tag,
 			       SD_MAX_VDI_TAG_LEN,
@@ -745,7 +750,7 @@ static int vdi_clone(int argc, char **argv)
 		goto out;
 
 	ret = do_vdi_create(dst_vdi, inode->vdi_size, base_vid, &new_vid, false,
-			    vdi_cmd_data.nr_copies, inode->copy_policy,
+			    inode->nr_copies, inode->copy_policy,
 			    inode->store_policy);
 	if (ret != EXIT_SUCCESS || !vdi_cmd_data.prealloc)
 		goto out;
@@ -1763,6 +1768,7 @@ static void vote_majority_object(struct vdi_check_info *info)
 	int count = 0, nr_live_copies = 0;
 	struct vdi_check_work *majority = NULL;
 
+	/* step 1 */
 	for (int i = 0; i < info->nr_copies; i++) {
 		struct vdi_check_work *vcw = &info->vcw[i];
 
@@ -1779,14 +1785,29 @@ static void vote_majority_object(struct vdi_check_info *info)
 			count--;
 	}
 
+	/* step 2 */
+	if (count > 0 && count <= nr_live_copies / 2) {
+		count = 0;
+		for (int i = 0; i < info->nr_copies; i++) {
+			struct vdi_check_work *vcw = &info->vcw[i];
+
+			if (!vcw->object_found)
+				continue;
+			if (!memcmp(majority->hash, vcw->hash,
+						sizeof(vcw->hash)))
+				count++;
+		}
+	}
+
 	if (!majority)
 		info->result = VDI_CHECK_NO_OBJ_FOUND;
-	else if (count < nr_live_copies / 2) {
+	else if (count > nr_live_copies / 2)
+		info->result = VDI_CHECK_SUCCESS;
+	else {
 		/* no majority found */
 		majority = NULL;
 		info->result = VDI_CHECK_NO_MAJORITY_FOUND;
-	} else
-		info->result = VDI_CHECK_SUCCESS;
+	}
 
 	info->majority = majority;
 }
