@@ -11,6 +11,7 @@
 #define DECLARE_BITMAP(name, bits) \
 	unsigned long name[BITS_TO_LONGS(bits)]
 #define BITS_PER_LONG (BITS_PER_BYTE * sizeof(long))
+#define BITS_PER_UINT64 (BITS_PER_BYTE * sizeof(uint64_t))
 
 #define __ffs(x)  (x ? __builtin_ffsl(x) - 1 : 0)
 #define ffz(x)  __ffs(~(x))
@@ -135,6 +136,11 @@ static inline void set_bit(int nr, unsigned long *addr)
 	addr[nr / BITS_PER_LONG] |= 1UL << (nr % BITS_PER_LONG);
 }
 
+static inline void set_bit_64(int nr, uint64_t *addr)
+{
+	addr[nr / BITS_PER_UINT64] |= 1ULL << (nr % BITS_PER_UINT64);
+}
+
 static inline void atomic_set_bit(int nr, unsigned long *addr)
 {
 	uatomic_or(addr + nr / BITS_PER_LONG, 1UL << (nr % BITS_PER_LONG));
@@ -162,7 +168,7 @@ static inline void clear_bit(unsigned int nr, unsigned long *addr)
  * set bit if value is nonzero. The last (most significant) bit is
  * at position 64.
  */
-#if __SIZEOF_LONG__ == 4
+#if SIZEOF_LONG == 4
 static __always_inline int fls64(uint64_t x)
 {
 	uint32_t h = x >> 32;
@@ -174,7 +180,7 @@ static __always_inline int fls64(uint64_t x)
 		return 64 - __builtin_clzl(h);
 	return 32 - __builtin_clzl(x);
 }
-#elif __SIZEOF_LONG__ == 8
+#elif SIZEOF_LONG == 8
 static __always_inline int fls64(uint64_t x)
 {
 	if (x == 0)
@@ -182,7 +188,7 @@ static __always_inline int fls64(uint64_t x)
 	return 64 - __builtin_clzl(x);
 }
 #else
-#error __SIZEOF_LONG__ not 4 or 8
+#error SIZEOF_LONG not 4 or 8
 #endif
 
 #endif /* __BITOPS_H__ */
