@@ -22,9 +22,16 @@ struct sd_option {
 	const char *help;
 };
 
+struct option_parser {
+	const char *option;
+	int (*parser)(const char *);
+};
+
 char *build_short_options(const struct sd_option *opts);
 struct option *build_long_options(const struct sd_option *opts);
 const char *option_get_help(const struct sd_option *, int);
+int option_parse(char *arg, const char *delim, struct option_parser *parsers);
+int option_parse_size(const char *value, uint64_t *ret);
 
 #define sd_for_each_option(opt, opts)		\
 	for (opt = (opts); opt->name; opt++)
