@@ -14,9 +14,11 @@
 #include <inttypes.h>
 #include <stdint.h>
 #include <stdbool.h>
+#include <string.h>
 #include <linux/limits.h>
 
 #include "compiler.h"
+#include "bitops.h"
 
 #define SD_PROTO_VER 0x02
 
@@ -72,14 +74,16 @@
 #define SD_RES_JOIN_FAILED   0x18 /* Target node had failed to join sheepdog */
 #define SD_RES_HALT          0x19 /* Sheepdog is stopped doing IO */
 #define SD_RES_READONLY      0x1A /* Object is read-only */
+#define SD_RES_BTREE_NOT_FOUND	0x1B /* Cannot found node in btree */
+#define SD_RES_BTREE_FOUND   0x1C /* Found node in btree */
+#define SD_RES_BTREE_REPEAT  0x1D /* Should repeat op in btree */
 
 /* errors above 0x80 are sheepdog-internal */
 
 /*
  * Object ID rules
  *
- *  0 - 19 (20 bits): data object space
- * 20 - 31 (12 bits): reserved data object space
+ *  0 - 31 (32 bits): data object space
  * 32 - 55 (24 bits): VDI object space
  * 56 - 59 ( 4 bits): reserved VDI object space
  * 60 - 63 ( 4 bits): object type indentifier space
@@ -90,7 +94,9 @@
 #define VDI_BIT (UINT64_C(1) << 63)
 #define VMSTATE_BIT (UINT64_C(1) << 62)
 #define VDI_ATTR_BIT (UINT64_C(1) << 61)
-#define MAX_DATA_OBJS (1ULL << 20)
+#define VDI_BTREE_BIT (UINT64_C(1) << 60)
+#define OLD_MAX_DATA_OBJS (1ULL << 20)
+#define MAX_DATA_OBJS (1ULL << 32)
 #define MAX_CHILDREN 1024U
 #define SD_MAX_VDI_LEN 256U
 #define SD_MAX_VDI_TAG_LEN 256U
@@ -99,11 +105,14 @@
 #define SD_MAX_SNAPSHOT_TAG_LEN 256U
 #define SD_NR_VDIS   (1U << 24)
 #define SD_DATA_OBJ_SIZE (UINT64_C(1) << 22)
+#define SD_OLD_MAX_VDI_SIZE (SD_DATA_OBJ_SIZE * OLD_MAX_DATA_OBJS)
 #define SD_MAX_VDI_SIZE (SD_DATA_OBJ_SIZE * MAX_DATA_OBJS)
 
 #define SD_INODE_SIZE (sizeof(struct sd_inode))
-#define SD_INODE_HEADER_SIZE (sizeof(struct sd_inode) - \
-			      sizeof(uint32_t) * MAX_DATA_OBJS)
+#define SD_INODE_INDEX_SIZE (sizeof(uint32_t) * MAX_DATA_OBJS)
+#define SD_INODE_DATA_INDEX (1ULL << 20)
+#define SD_INODE_DATA_INDEX_SIZE (sizeof(uint32_t) * SD_INODE_DATA_INDEX)
+#define SD_INODE_HEADER_SIZE offsetof(struct sd_inode, data_vdi_id)
 #define SD_ATTR_OBJ_SIZE (sizeof(struct sheepdog_vdi_attr))
 #define CURRENT_VDI_ID 0
 
@@ -123,14 +132,21 @@ struct sd_req {
 		struct {
 			uint64_t	oid;
 			uint64_t	cow_oid;
-			uint32_t	copies;
+			uint8_t		copies;
+			uint8_t		copy_policy;
+			uint8_t		ec_index;
+			uint8_t		reserved;
 			uint32_t	tgt_epoch;
-			uint64_t	offset;
+			uint32_t	offset;
+			uint32_t	__pad;
 		} obj;
 		struct {
 			uint64_t	vdi_size;
 			uint32_t	base_vdi_id;
-			uint32_t	copies;
+			uint8_t		copies;
+			uint8_t		copy_policy;
+			uint8_t		store_policy;
+			uint8_t		reserved;
 			uint32_t	snapid;
 		} vdi;
 
@@ -138,15 +154,18 @@ struct sd_req {
 		struct {
 			uint64_t	oid;
 			uint64_t	ctime;
-			uint32_t	copies;
+			uint8_t		copies;
+			uint8_t		copy_policy;
+			uint16_t	flags;
 			uint32_t	tag;
 		} cluster;
 		struct {
 			uint32_t	old_vid;
 			uint32_t	new_vid;
-			uint32_t	copies;
+			uint8_t		copies;
 			uint8_t		set_bitmap; /* 0 means false */
 						    /* others mean true */
+			uint8_t		copy_policy;
 		} vdi_state;
 
 		uint32_t		__pad[8];
@@ -164,7 +183,8 @@ struct sd_rsp {
 		uint32_t        result;
 		struct {
 			uint32_t	__pad;
-			uint32_t	copies;
+			uint8_t		copies;
+			uint8_t		reserved[3];
 			uint64_t	offset;
 		} obj;
 		struct {
@@ -172,15 +192,15 @@ struct sd_rsp {
 			uint32_t	rsvd;
 			uint32_t	vdi_id;
 			uint32_t	attr_id;
-			uint32_t	copies;
+			uint8_t		copies;
+			uint8_t		reserved[3];
 		} vdi;
 
 		/* sheepdog-internal */
 		struct {
 			uint32_t	__pad;
 			uint32_t	nr_nodes;
-			uint32_t	local_idx;
-			uint32_t	__reserved;
+			uint32_t	__reserved[2];
 			uint64_t	store_size;
 			uint64_t	store_free;
 		} node;
@@ -202,16 +222,48 @@ struct sd_inode {
 	uint64_t vm_clock_nsec;
 	uint64_t vdi_size;
 	uint64_t vm_state_size;
-	uint16_t copy_policy;
+	uint8_t  copy_policy;
+	uint8_t  store_policy;
 	uint8_t  nr_copies;
 	uint8_t  block_size_shift;
 	uint32_t snap_id;
 	uint32_t vdi_id;
 	uint32_t parent_vdi_id;
 	uint32_t child_vdi_id[MAX_CHILDREN];
-	uint32_t data_vdi_id[MAX_DATA_OBJS];
+	uint32_t data_vdi_id[SD_INODE_DATA_INDEX];
+	uint32_t btree_counter;
 };
 
+struct sd_extent {
+	uint32_t idx;
+	uint32_t vdi_id;
+};
+
+struct sd_extent_idx {
+	uint32_t idx;
+	uint64_t oid;
+};
+
+#define INODE_BTREE_MAGIC	0x6274
+
+struct sd_extent_header {
+	uint16_t magic;
+	uint16_t depth;	/* 1 -- ext node; 2 -- idx node */
+	uint32_t entries;
+};
+
+enum btree_node_type {
+	BTREE_HEAD = 1,
+	BTREE_EXT,
+	BTREE_IDX,
+};
+
+typedef int (*write_node_fn)(uint64_t id, void *mem, unsigned int len,
+				uint64_t offset, uint32_t flags, int copies,
+				int copy_policy, bool create, bool direct);
+typedef int (*read_node_fn)(uint64_t id, void **mem, unsigned int len,
+				uint64_t offset);
+
 struct sheepdog_vdi_attr {
 	char name[SD_MAX_VDI_LEN];
 	char tag[SD_MAX_VDI_TAG_LEN];
@@ -222,33 +274,112 @@ struct sheepdog_vdi_attr {
 	char value[SD_MAX_VDI_ATTR_VALUE_LEN];
 };
 
+extern void sd_inode_init(void *data, int depth);
+extern uint32_t sd_inode_get_vid(read_node_fn reader,
+				 const struct sd_inode *inode, uint32_t idx);
+extern void sd_inode_set_vid(write_node_fn writer, read_node_fn reader,
+			     struct sd_inode *inode, uint32_t idx,
+			     uint32_t vdi_id);
+extern int sd_inode_write(write_node_fn writer, struct sd_inode *inode,
+			  int flags, bool create, bool direct);
+extern int sd_inode_write_vid(write_node_fn writer, struct sd_inode *inode,
+			      uint32_t idx, uint32_t vid, uint32_t value,
+			      int flags, bool create, bool direct);
+extern uint32_t sd_inode_get_meta_size(struct sd_inode *inode, size_t size);
+extern void sd_inode_copy_vdis(write_node_fn writer, read_node_fn reader,
+			       uint32_t *data_vdi_id, uint8_t store_policy,
+			       uint8_t nr_copies, uint8_t copy_policy,
+			       struct sd_inode *newi);
+
+typedef void (*btree_cb_fn)(void *data, enum btree_node_type type, void *arg);
+extern void traverse_btree(read_node_fn reader, const struct sd_inode *inode,
+			   btree_cb_fn fn, void *arg);
+
 /* 64 bit FNV-1a non-zero initial basis */
 #define FNV1A_64_INIT ((uint64_t) 0xcbf29ce484222325ULL)
+#define FNV_64_PRIME ((uint64_t) 0x100000001b3ULL)
 
 /* 64 bit Fowler/Noll/Vo FNV-1a hash code */
 static inline uint64_t fnv_64a_buf(const void *buf, size_t len, uint64_t hval)
 {
-	unsigned char *bp = (unsigned char *) buf;
-	unsigned char *be = bp + len;
-	while (bp < be) {
-		hval ^= (uint64_t) *bp++;
-		hval += (hval << 1) + (hval << 4) + (hval << 5) +
-			(hval << 7) + (hval << 8) + (hval << 40);
+	const unsigned char *p = (const unsigned char *) buf;
+
+	for (int i = 0; i < len; i++) {
+		hval ^= (uint64_t) p[i];
+		hval *= FNV_64_PRIME;
 	}
+
 	return hval;
 }
 
+/*
+ * The result is same as fnv_64a_buf(&oid, sizeof(oid), hval) but this function
+ * is a bit faster.
+ */
+static inline uint64_t fnv_64a_64(uint64_t oid, uint64_t hval)
+{
+	hval ^= oid & 0xff;
+	hval *= FNV_64_PRIME;
+	hval ^= oid >> 8 & 0xff;
+	hval *= FNV_64_PRIME;
+	hval ^= oid >> 16 & 0xff;
+	hval *= FNV_64_PRIME;
+	hval ^= oid >> 24 & 0xff;
+	hval *= FNV_64_PRIME;
+	hval ^= oid >> 32 & 0xff;
+	hval *= FNV_64_PRIME;
+	hval ^= oid >> 40 & 0xff;
+	hval *= FNV_64_PRIME;
+	hval ^= oid >> 48 & 0xff;
+	hval *= FNV_64_PRIME;
+	hval ^= oid >> 56 & 0xff;
+	hval *= FNV_64_PRIME;
+
+	return hval;
+}
+
+static inline uint64_t sd_hash(const void *buf, size_t len)
+{
+	uint64_t hval = fnv_64a_buf(buf, len, FNV1A_64_INIT);
+
+	return fnv_64a_64(hval, hval);
+}
+
+static inline uint64_t sd_hash_64(uint64_t oid)
+{
+	uint64_t hval = fnv_64a_64(oid, FNV1A_64_INIT);
+
+	return fnv_64a_64(hval, hval);
+}
+
+static inline uint64_t sd_hash_next(uint64_t hval)
+{
+	return fnv_64a_64(hval, hval);
+}
+
+/*
+ * Create a hash value from an object id.  The result is same as sd_hash(&oid,
+ * sizeof(oid)) but this function is a bit faster.
+ */
+static inline uint64_t sd_hash_oid(uint64_t oid)
+{
+	return sd_hash_64(oid);
+}
+
+/*
+ * Create a hash value from a vdi name.  We cannot use sd_hash_buf for this
+ * purpose because of backward compatibility.
+ */
+static inline uint32_t sd_hash_vdi(const char *name)
+{
+	uint64_t hval = fnv_64a_buf(name, strlen(name), FNV1A_64_INIT);
+
+	return (uint32_t)(hval & (SD_NR_VDIS - 1));
+}
+
 static inline uint64_t hash_64(uint64_t val, unsigned int bits)
 {
-	uint64_t hash = fnv_64a_buf(&val, sizeof(uint64_t), FNV1A_64_INIT);
-
-	return hash & ((1 << bits) - 1);
-}
-
-static inline bool is_data_obj_writeable(const struct sd_inode *inode,
-					 int idx)
-{
-	return inode->vdi_id == inode->data_vdi_id[idx];
+	return sd_hash_64(val) >> (64 - bits);
 }
 
 static inline bool is_vdi_obj(uint64_t oid)
@@ -266,10 +397,20 @@ static inline bool is_vdi_attr_obj(uint64_t oid)
 	return !!(oid & VDI_ATTR_BIT);
 }
 
+static inline bool is_vdi_btree_obj(uint64_t oid)
+{
+	return !!(oid & VDI_BTREE_BIT);
+}
+
 static inline bool is_data_obj(uint64_t oid)
 {
 	return !is_vdi_obj(oid) && !is_vmstate_obj(oid) &&
-		!is_vdi_attr_obj(oid);
+		!is_vdi_attr_obj(oid) && !is_vdi_btree_obj(oid);
+}
+
+static inline size_t count_data_objs(const struct sd_inode *inode)
+{
+	return DIV_ROUND_UP(inode->vdi_size, SD_DATA_OBJ_SIZE);
 }
 
 static inline size_t get_objsize(uint64_t oid)
@@ -280,6 +421,9 @@ static inline size_t get_objsize(uint64_t oid)
 	if (is_vdi_attr_obj(oid))
 		return SD_ATTR_OBJ_SIZE;
 
+	if (is_vdi_btree_obj(oid))
+		return SD_INODE_DATA_INDEX_SIZE;
+
 	return SD_DATA_OBJ_SIZE;
 }
 
@@ -293,7 +437,7 @@ static inline uint64_t vid_to_vdi_oid(uint32_t vid)
 	return VDI_BIT | ((uint64_t)vid << VDI_SPACE_SHIFT);
 }
 
-static inline uint64_t vid_to_data_oid(uint32_t vid, uint32_t idx)
+static inline uint64_t vid_to_data_oid(uint32_t vid, uint64_t idx)
 {
 	return ((uint64_t)vid << VDI_SPACE_SHIFT) | idx;
 }
@@ -308,6 +452,11 @@ static inline uint64_t vid_to_attr_oid(uint32_t vid, uint32_t attrid)
 	return ((uint64_t)vid << VDI_SPACE_SHIFT) | VDI_ATTR_BIT | attrid;
 }
 
+static inline uint64_t vid_to_btree_oid(uint32_t vid, uint32_t btreeid)
+{
+	return ((uint64_t)vid << VDI_SPACE_SHIFT) | VDI_BTREE_BIT | btreeid;
+}
+
 static inline uint64_t vid_to_vmstate_oid(uint32_t vid, uint32_t idx)
 {
 	return VMSTATE_BIT | ((uint64_t)vid << VDI_SPACE_SHIFT) | idx;
