@@ -9,7 +9,7 @@ void sockfd_cache_put(const struct node_id *nid, struct sockfd *sfd);
 void sockfd_cache_del_node(const struct node_id *nid);
 void sockfd_cache_del(const struct node_id *nid, struct sockfd *sfd);
 void sockfd_cache_add(const struct node_id *nid);
-void sockfd_cache_add_group(const struct sd_node *nodes, int nr);
+void sockfd_cache_add_group(const struct rb_root *nroot);
 
 int sockfd_init(void);
 
