@@ -11,7 +11,7 @@ struct work;
 typedef void (*work_func_t)(struct work *);
 
 struct work {
-	struct list_head w_list;
+	struct list_node w_list;
 	work_func_t fn;
 	work_func_t done;
 };
@@ -61,9 +61,12 @@ static inline bool is_worker_thread(void)
 int init_work_queue(size_t (*get_nr_nodes)(void));
 struct work_queue *create_work_queue(const char *name, enum wq_thread_control);
 struct work_queue *create_ordered_work_queue(const char *name);
-void suspend_worker_threads(void);
-void resume_worker_threads(void);
 void queue_work(struct work_queue *q, struct work *work);
 bool work_queue_empty(struct work_queue *q);
 
+#ifdef HAVE_TRACE
+void suspend_worker_threads(void);
+void resume_worker_threads(void);
+#endif	/* HAVE_TRACE */
+
 #endif
