@@ -171,7 +171,7 @@ static int server_log_formatter(char *buff, size_t size,
 {
 	char *p = buff;
 	struct tm tm;
-	size_t len;
+	ssize_t len;
 	char thread_name[MAX_THREAD_NAME_LEN];
 
 	if (print_time) {
@@ -193,7 +193,7 @@ static int server_log_formatter(char *buff, size_t size,
 		       msg->str, colorize ? TEXT_NORMAL : "");
 	if (len < 0)
 		len = 0;
-	p += min(len, size - 1);
+	p += min((size_t)len, size - 1);
 
 	return p - buff;
 }
@@ -212,7 +212,7 @@ static int json_log_formatter(char *buff, size_t size,
 			      const struct logmsg *msg, bool print_time)
 {
 	char *p = buff;
-	size_t len;
+	ssize_t len;
 
 	assert(logger_user_info);
 
@@ -229,7 +229,7 @@ static int json_log_formatter(char *buff, size_t size,
 		       msg->worker_idx, msg->func, msg->line);
 	if (len < 0)
 		return 0;
-	len = min(len, size - 1);
+	len = min((size_t)len, size - 1);
 	p += len;
 	size -= len;
 
@@ -711,13 +711,27 @@ int log_init(const char *program_name, enum log_dst_type type, int level,
 
 void log_close(void)
 {
-	if (la) {
-		la->active = false;
-		waitpid(logger_pid, NULL, 0);
+	pid_t pid;
 
-		syslog(LOG_WARNING, "logger pid %d stopped\n", logger_pid);
-		closelog();
-		free_logarea();
+	if (!la)
+		return;
+
+	while (true) {
+		la->active = false;
+		pid = waitpid(logger_pid, NULL, WNOHANG);
+		if (pid == 0) {
+			usleep(100000);
+			continue;
+		} else if (pid > 0) {
+			syslog(LOG_WARNING, "logger pid %d stopped\n",
+					logger_pid);
+			closelog();
+			free_logarea();
+			break;
+		} else {
+			syslog(LOG_ERR, "waitpid() failure\n");
+			exit(1);
+		}
 	}
 }
 
