@@ -61,59 +61,6 @@ int conn_rx_on(struct connection *conn)
 	return modify_event(conn->fd, conn->events);
 }
 
-bool is_conn_dead(const struct connection *conn)
-{
-	if (conn->c_rx_state == C_IO_CLOSED || conn->c_tx_state == C_IO_CLOSED)
-		return true;
-	else
-		return false;
-}
-
-int rx(struct connection *conn, enum conn_state next_state)
-{
-	int ret;
-
-	ret = read(conn->fd, conn->rx_buf, conn->rx_length);
-	if (!ret) {
-		conn->c_rx_state = C_IO_CLOSED;
-		return 0;
-	}
-
-	if (ret < 0) {
-		if (errno != EAGAIN && errno != EINTR)
-			conn->c_rx_state = C_IO_CLOSED;
-		return 0;
-	}
-
-	conn->rx_length -= ret;
-	conn->rx_buf = (char *)conn->rx_buf + ret;
-
-	if (!conn->rx_length)
-		conn->c_rx_state = next_state;
-
-	return ret;
-}
-
-int tx(struct connection *conn, enum conn_state next_state)
-{
-	int ret;
-
-	ret = write(conn->fd, conn->tx_buf, conn->tx_length);
-	if (ret < 0) {
-		if (errno != EAGAIN && errno != EINTR)
-			conn->c_tx_state = C_IO_CLOSED;
-		return 0;
-	}
-
-	conn->tx_length -= ret;
-	conn->tx_buf = (char *)conn->tx_buf + ret;
-
-	if (!conn->tx_length)
-		conn->c_tx_state = next_state;
-
-	return ret;
-}
-
 int create_listen_ports(const char *bindaddr, int port,
 		int (*callback)(int fd, void *), void *data)
 {
@@ -170,12 +117,6 @@ int create_listen_ports(const char *bindaddr, int port,
 			continue;
 		}
 
-		ret = set_nonblocking(fd);
-		if (ret < 0) {
-			close(fd);
-			continue;
-		}
-
 		ret = callback(fd, data);
 		if (ret) {
 			close(fd);
@@ -276,7 +217,7 @@ int do_read(int sockfd, void *buf, int len, bool (*need_retry)(uint32_t epoch),
 reread:
 	ret = read(sockfd, buf, len);
 	if (ret == 0) {
-		sd_err("connection is closed (%d bytes left)", len);
+		sd_debug("connection is closed (%d bytes left)", len);
 		return 1;
 	}
 	if (ret < 0) {
@@ -483,23 +424,6 @@ uint8_t *str_to_addr(const char *ipstr, uint8_t *addr)
 	return addr;
 }
 
-int set_nonblocking(int fd)
-{
-	int ret;
-
-	ret = fcntl(fd, F_GETFL);
-	if (ret < 0) {
-		sd_err("fcntl F_GETFL failed: %m");
-		close(fd);
-	} else {
-		ret = fcntl(fd, F_SETFL, ret | O_NONBLOCK);
-		if (ret < 0)
-			sd_err("fcntl O_NONBLOCK failed: %m");
-	}
-
-	return ret;
-}
-
 int set_snd_timeout(int fd)
 {
 	struct timeval timeout;
@@ -592,12 +516,12 @@ int get_local_addr(uint8_t *bytes)
 			memset(bytes, 0, 12);
 			memcpy(bytes + 12, &sin->sin_addr, 4);
 			memcpy(bytes + 12, &sin->sin_addr, 4);
-			sd_err("found IPv4 address");
+			sd_notice("found IPv4 address");
 			goto out;
 		case AF_INET6:
 			sin6 = (struct sockaddr_in6 *)ifa->ifa_addr;
 			memcpy(bytes, &sin6->sin6_addr, 16);
-			sd_err("found IPv6 address");
+			sd_notice("found IPv6 address");
 			goto out;
 		}
 	}
@@ -636,10 +560,6 @@ int create_unix_domain_socket(const char *unix_path,
 		goto err;
 	}
 
-	ret = set_nonblocking(fd);
-	if (ret < 0)
-		goto err;
-
 	ret = callback(fd, data);
 	if (ret)
 		goto err;
