@@ -1734,8 +1734,8 @@ sub process {
 				     "[BCS] don't use block comments for one liner comment\n" . $hereprev);
 			} else {
 				# case E
-				my $ln = $linenr;
-				while ($rawlines[$ln] !~ /^\+\s*\/\*/ && $ln >= 0) { $ln--; }
+				my $ln = $linenr - 1;
+				while ($rawlines[$ln] =~ /^\+/ && $rawlines[$ln] !~ /^\+\s*\/\*/ && $ln >= 0) { $ln--; }
 				if ($rawlines[$ln] =~ /^\+\s*\/\*./) {
 					WARN("BLOCK_COMMENT_STYLE",
 					     "[BCS] don't comment at first line in block comments\n" . $hereprev);
@@ -2812,29 +2812,6 @@ sub process {
 			}
 		}
 
-# typecasts on min/max could be min_t/max_t
-		if (defined $stat &&
-		    $stat =~ /^\+(?:.*?)\b(min|max)\s*\(\s*$FuncArg\s*,\s*$FuncArg\s*\)/) {
-			if (defined $2 || defined $8) {
-				my $call = $1;
-				my $cast1 = deparenthesize($2);
-				my $arg1 = $3;
-				my $cast2 = deparenthesize($8);
-				my $arg2 = $9;
-				my $cast;
-
-				if ($cast1 ne "" && $cast2 ne "") {
-					$cast = "$cast1 or $cast2";
-				} elsif ($cast1 ne "") {
-					$cast = $cast1;
-				} else {
-					$cast = $cast2;
-				}
-				WARN("MINMAX",
-				     "$call() should probably be ${call}_t($cast, $arg1, $arg2)\n" . "$here\n$stat\n");
-			}
-		}
-
 # check for new externs in .c files.
 		if ($realfile =~ /\.c$/ && defined $stat &&
 		    $stat =~ /^.\s*(?:extern\s+)?$Type\s+($Ident)(\s*)\(/s)
