@@ -20,6 +20,7 @@
 #include "work.h"
 
 #define CPG_INIT_RETRY_CNT 10
+#define COROSYNC_MAX_NODES 1024
 
 struct cpg_node {
 	uint32_t nodeid;
@@ -36,7 +37,7 @@ static struct cpg_node this_node;
 static LIST_HEAD(corosync_block_event_list);
 static LIST_HEAD(corosync_nonblock_event_list);
 
-static struct cpg_node cpg_nodes[SD_MAX_NODES];
+static struct cpg_node cpg_nodes[COROSYNC_MAX_NODES];
 static size_t nr_cpg_nodes;
 static bool self_elect;
 static bool join_finished;
@@ -72,11 +73,11 @@ struct corosync_event {
 	size_t msg_len;
 
 	uint32_t nr_nodes;
-	struct cpg_node nodes[SD_MAX_NODES];
+	struct cpg_node nodes[COROSYNC_MAX_NODES];
 
 	bool callbacked;
 
-	struct list_head list;
+	struct list_node list;
 };
 
 struct corosync_message {
@@ -84,7 +85,7 @@ struct corosync_message {
 	enum corosync_message_type type:16;
 	uint16_t nr_nodes;
 	uint32_t msg_len;
-	struct cpg_node nodes[SD_MAX_NODES];
+	struct cpg_node nodes[COROSYNC_MAX_NODES];
 	uint8_t msg[0];
 };
 
@@ -167,9 +168,11 @@ static int send_message(enum corosync_message_type type,
 {
 	struct iovec iov[2];
 	int ret, iov_cnt = 1;
+	size_t mlen = MIN(msg_len, SD_MAX_EVENT_BUF_SIZE);
+
 	struct corosync_message cmsg = {
 		.type = type,
-		.msg_len = msg_len,
+		.msg_len = mlen,
 		.sender = *sender,
 		.nr_nodes = nr_nodes,
 	};
@@ -181,7 +184,7 @@ static int send_message(enum corosync_message_type type,
 	iov[0].iov_len = sizeof(cmsg);
 	if (msg) {
 		iov[1].iov_base = msg;
-		iov[1].iov_len = msg_len;
+		iov[1].iov_len = mlen;
 		iov_cnt++;
 	}
 retry:
@@ -237,13 +240,11 @@ find_event(enum corosync_event_type type, struct cpg_node *sender)
 		return find_nonblock_event(type, sender);
 }
 
-static void build_node_list(const struct cpg_node *nodes, size_t nr_nodes,
-			    struct sd_node *entries)
+static void build_node_list(struct cpg_node *nodes, size_t nr_nodes,
+			    struct rb_root *nroot)
 {
-	int i;
-
-	for (i = 0; i < nr_nodes; i++)
-		entries[i] = nodes[i].node;
+	for (int i = 0; i < nr_nodes; i++)
+		rb_insert(nroot, &nodes[i].node, rb, node_cmp);
 }
 
 /*
@@ -253,8 +254,9 @@ static void build_node_list(const struct cpg_node *nodes, size_t nr_nodes,
  */
 static bool __corosync_dispatch_one(struct corosync_event *cevent)
 {
-	struct sd_node entries[SD_MAX_NODES], *node;
+	struct sd_node *node;
 	struct cpg_node *n;
+	struct rb_root nroot = RB_ROOT;
 	int idx;
 
 	switch (cevent->type) {
@@ -267,8 +269,8 @@ static bool __corosync_dispatch_one(struct corosync_event *cevent)
 			/* sd_join_handler() must be called only once */
 			return false;
 
-		build_node_list(cpg_nodes, nr_cpg_nodes, entries);
-		if (sd_join_handler(&cevent->sender.node, entries,
+		build_node_list(cpg_nodes, nr_cpg_nodes, &nroot);
+		if (sd_join_handler(&cevent->sender.node, &nroot,
 				    nr_cpg_nodes, cevent->msg)) {
 			send_message(COROSYNC_MSG_TYPE_ACCEPT, &cevent->sender,
 				     cpg_nodes, nr_cpg_nodes, cevent->msg,
@@ -281,8 +283,8 @@ static bool __corosync_dispatch_one(struct corosync_event *cevent)
 		add_cpg_node(cpg_nodes, nr_cpg_nodes, &cevent->sender);
 		nr_cpg_nodes++;
 
-		build_node_list(cpg_nodes, nr_cpg_nodes, entries);
-		sd_accept_handler(&cevent->sender.node, entries, nr_cpg_nodes,
+		build_node_list(cpg_nodes, nr_cpg_nodes, &nroot);
+		sd_accept_handler(&cevent->sender.node, &nroot, nr_cpg_nodes,
 				  cevent->msg);
 		break;
 	case COROSYNC_EVENT_TYPE_LEAVE:
@@ -294,8 +296,8 @@ static bool __corosync_dispatch_one(struct corosync_event *cevent)
 
 		del_cpg_node(cpg_nodes, nr_cpg_nodes, &cevent->sender);
 		nr_cpg_nodes--;
-		build_node_list(cpg_nodes, nr_cpg_nodes, entries);
-		sd_leave_handler(&cevent->sender.node, entries, nr_cpg_nodes);
+		build_node_list(cpg_nodes, nr_cpg_nodes, &nroot);
+		sd_leave_handler(&cevent->sender.node, &nroot, nr_cpg_nodes);
 		break;
 	case COROSYNC_EVENT_TYPE_BLOCK:
 		if (cevent->callbacked)
@@ -544,9 +546,9 @@ static void cdrv_cpg_confchg(cpg_handle_t handle,
 {
 	struct corosync_event *cevent;
 	int i;
-	struct cpg_node member_sheep[SD_MAX_NODES];
-	struct cpg_node joined_sheep[SD_MAX_NODES];
-	struct cpg_node left_sheep[SD_MAX_NODES];
+	struct cpg_node member_sheep[COROSYNC_MAX_NODES];
+	struct cpg_node joined_sheep[COROSYNC_MAX_NODES];
+	struct cpg_node left_sheep[COROSYNC_MAX_NODES];
 	bool promote = true;
 
 	sd_debug("mem:%zu, joined:%zu, left:%zu", member_list_entries,
@@ -640,7 +642,6 @@ static int corosync_join(const struct sd_node *myself,
 			 void *opaque, size_t opaque_len)
 {
 	int ret;
-
 retry:
 	ret = cpg_join(cpg_handle, &cpg_group);
 	switch (ret) {
@@ -773,6 +774,14 @@ again:
 	return 0;
 }
 
+static void corosync_lock(uint64_t lock_id)
+{
+}
+
+static void corosync_unlock(uint64_t lock_id)
+{
+}
+
 static int corosync_update_node(struct sd_node *node)
 {
 	struct cpg_node cnode = this_node;
@@ -793,6 +802,8 @@ static struct cluster_driver cdrv_corosync = {
 	.notify		= corosync_notify,
 	.block		= corosync_block,
 	.unblock	= corosync_unblock,
+	.lock		= corosync_lock,
+	.unlock		= corosync_unlock,
 	.update_node	= corosync_update_node,
 };
 
