@@ -19,7 +19,6 @@
 #include <sys/types.h>
 #include <sys/un.h>
 #include <sys/epoll.h>
-#include <sys/eventfd.h>
 
 #include "cluster.h"
 #include "event.h"
@@ -172,7 +171,7 @@ struct sph_event {
 
 	bool callbacked, removed;
 
-	struct list_head event_list;
+	struct list_node event_list;
 };
 
 static LIST_HEAD(nonblocked_event_list);
@@ -241,8 +240,6 @@ static void push_sph_event(bool nonblock, struct sd_node *sender,
 	ev->removed = false;
 	ev->callbacked = false;
 
-	INIT_LIST_HEAD(&ev->event_list);
-
 	if (nonblock)
 		list_add_tail(&ev->event_list, &nonblocked_event_list);
 	else
