@@ -16,6 +16,7 @@
 #include <sys/epoll.h>
 #include <zookeeper/zookeeper.h>
 #include <pthread.h>
+#include <semaphore.h>
 
 #include "cluster.h"
 #include "config.h"
@@ -29,10 +30,11 @@
 #define BASE_ZNODE "/sheepdog"
 #define QUEUE_ZNODE BASE_ZNODE "/queue"
 #define MEMBER_ZNODE BASE_ZNODE "/member"
-#define MASTER_ZNONE BASE_ZNODE "/master"
+#define MASTER_ZNODE BASE_ZNODE "/master"
 #define LOCK_ZNODE BASE_ZNODE "/lock"
 
 static int zk_timeout = SESSION_TIMEOUT;
+static int my_master_seq;
 
 /* structure for distributed lock */
 struct cluster_lock {
@@ -42,8 +44,7 @@ struct cluster_lock {
 	/* referenced by different threads in one sheepdog daemon */
 	uint64_t ref;
 	/* wait for the release of id by other lock owner */
-	struct sd_cond wait_wakeup;
-	struct sd_mutex wait_mutex;
+	sem_t wait_wakeup;
 	/* lock for different threads of the same node on the same id */
 	struct sd_mutex id_lock;
 	char lock_path[MAX_NODE_STR_LEN];
@@ -315,7 +316,7 @@ static int lock_table_lookup_wakeup(uint64_t lock_id)
 	sd_mutex_lock(table_locks + hval);
 	hlist_for_each_entry(lock, iter, cluster_locks_table + hval, hnode) {
 		if (lock->id == lock_id) {
-			sd_cond_signal(&lock->wait_wakeup);
+			sem_post(&lock->wait_wakeup);
 			res = 0;
 			break;
 		}
@@ -352,8 +353,7 @@ static struct cluster_lock *lock_table_lookup_acquire(uint64_t lock_id)
 		if (rc)
 			panic("Failed to init node %s", path);
 
-		sd_cond_init(&ret_lock->wait_wakeup);
-		sd_init_mutex(&ret_lock->wait_mutex);
+		sem_init(&ret_lock->wait_wakeup, 0, 1);
 		sd_init_mutex(&ret_lock->id_lock);
 
 		hlist_add_head(&(ret_lock->hnode), cluster_locks_table + hval);
@@ -398,8 +398,7 @@ static void lock_table_lookup_release(uint64_t lock_id)
 			hlist_del(iter);
 			/* free all resource used by this lock */
 			sd_destroy_mutex(&lock->id_lock);
-			sd_destroy_mutex(&lock->wait_mutex);
-			sd_destroy_cond(&lock->wait_wakeup);
+			sem_destroy(&lock->wait_wakeup);
 			snprintf(path, MAX_NODE_STR_LEN, LOCK_ZNODE "/%"PRIu64,
 				 lock->id);
 			/*
@@ -643,7 +642,7 @@ static inline void build_node_list(void)
 static int zk_queue_init(void)
 {
 	RETURN_IF_ERROR(zk_init_node(BASE_ZNODE), "path %s", BASE_ZNODE);
-	RETURN_IF_ERROR(zk_init_node(MASTER_ZNONE), "path %s", MASTER_ZNONE);
+	RETURN_IF_ERROR(zk_init_node(MASTER_ZNODE), "path %s", MASTER_ZNODE);
 	RETURN_IF_ERROR(zk_init_node(QUEUE_ZNODE), "path %s", QUEUE_ZNODE);
 	RETURN_IF_ERROR(zk_init_node(MEMBER_ZNODE), "path %s", MEMBER_ZNODE);
 	return ZOK;
@@ -670,6 +669,7 @@ static int add_event(enum zk_event_type type, struct zk_node *znode, void *buf,
 	struct zk_event ev;
 	int rc;
 
+	memset(&ev, 0, sizeof(ev));
 	ev.id = get_uniq_id();
 	ev.type = type;
 	ev.sender = *znode;
@@ -723,7 +723,7 @@ static void zk_watcher(zhandle_t *zh, int type, int state, const char *path,
 			return;
 		}
 
-		ret = sscanf(path, MASTER_ZNONE "/%s", str);
+		ret = sscanf(path, MASTER_ZNODE "/%s", str);
 		if (ret == 1) {
 			zk_compete_master();
 			return;
@@ -817,17 +817,17 @@ static int zk_find_master(int *master_seq, char *master_name)
 	char master_compete_path[MAX_NODE_STR_LEN];
 
 	if (*master_seq < 0) {
-		RETURN_IF_ERROR(zk_get_least_seq(MASTER_ZNONE,
+		RETURN_IF_ERROR(zk_get_least_seq(MASTER_ZNODE,
 						 master_compete_path,
 						 MAX_NODE_STR_LEN, master_name,
 						 &len), "");
-		sscanf(master_compete_path, MASTER_ZNONE "/%"PRId32,
+		sscanf(master_compete_path, MASTER_ZNODE "/%"PRId32,
 		       master_seq);
 		return ZOK;
 	} else {
 		while (true) {
 			snprintf(master_compete_path, len,
-				 MASTER_ZNONE "/%010"PRId32, *master_seq);
+				 MASTER_ZNODE "/%010"PRId32, *master_seq);
 			rc = zk_get_data(master_compete_path, master_name,
 					 &len);
 			switch (rc) {
@@ -856,7 +856,7 @@ static int zk_verify_last_sheep_join(int seq, int *last_sheep)
 	char path[MAX_NODE_STR_LEN], name[MAX_NODE_STR_LEN];
 
 	for (*last_sheep = seq - 1; *last_sheep >= 0; (*last_sheep)--) {
-		snprintf(path, MAX_NODE_STR_LEN, MASTER_ZNONE "/%010"PRId32,
+		snprintf(path, MAX_NODE_STR_LEN, MASTER_ZNODE "/%010"PRId32,
 			 *last_sheep);
 		rc = zk_get_data(path, name, &len);
 		switch (rc) {
@@ -914,18 +914,18 @@ static void zk_compete_master(void)
 			if (uatomic_is_true(&stop))
 				goto out_unlock;
 			/* duplicate sequential node has no side-effect */
-			rc = zk_create_seq_node(MASTER_ZNONE "/",
+			rc = zk_create_seq_node(MASTER_ZNODE "/",
 						node_to_str(&this_node.node),
 						MAX_NODE_STR_LEN,
 						my_compete_path,
 						MAX_NODE_STR_LEN, true);
 		} while (rc == ZOPERATIONTIMEOUT || rc == ZCONNECTIONLOSS);
-		CHECK_ZK_RC(rc, MASTER_ZNONE "/");
+		CHECK_ZK_RC(rc, MASTER_ZNODE "/");
 		if (rc != ZOK)
 			goto out_unlock;
 
 		sd_debug("my compete path: %s", my_compete_path);
-		sscanf(my_compete_path, MASTER_ZNONE "/%"PRId32,
+		sscanf(my_compete_path, MASTER_ZNODE "/%"PRId32,
 		       &my_seq);
 	}
 
@@ -953,6 +953,7 @@ static void zk_compete_master(void)
 	}
 success:
 	uatomic_set_true(&is_master);
+	my_master_seq = master_seq;
 	sd_debug("success");
 out_unlock:
 	sd_rw_unlock(&zk_compete_master_lock);
@@ -970,7 +971,8 @@ static int zk_join(const struct sd_node *myself,
 	rc = zk_node_exists(path);
 	if (rc == ZOK) {
 		sd_err("Previous zookeeper session exist, shoot myself. Please "
-		       "wait for %d seconds to join me again.", zk_timeout);
+			"wait for %d seconds to join me again.",
+			DIV_ROUND_UP(zk_timeout, 1000));
 		exit(1);
 	}
 
@@ -987,6 +989,12 @@ static int zk_leave(void)
 	sd_info("leaving from cluster");
 	uatomic_set_true(&stop);
 
+	if (uatomic_is_true(&is_master)) {
+		snprintf(path, sizeof(path), MASTER_ZNODE "/%010"PRId32,
+				my_master_seq);
+		zk_delete_node(path, -1);
+	}
+
 	snprintf(path, sizeof(path), MEMBER_ZNODE"/%s",
 		 node_to_str(&this_node.node));
 	add_event(EVENT_LEAVE, &this_node, NULL, 0);
@@ -1267,7 +1275,7 @@ kick_block_event:
  * This operation will create a seq-ephemeral znode in lock directory
  * of zookeeper (use lock-id as dir name). The smallest file path in
  * this directory wil be the owner of the lock; the other threads will
- * wait on a struct sd_cond (cluster_lock->wait_wakeup)
+ * wait on a sem_t (cluster_lock->wait_wakeup)
  */
 static void zk_lock(uint64_t lock_id)
 {
@@ -1334,9 +1342,7 @@ create_seq_node:
 		if (rc == ZOK) {
 			sd_debug("call zoo_exists success %s", lowest_seq_path);
 			/* Use wait_timeout to avoid missing wakeup signal */
-			sd_cond_wait_timeout(&cluster_lock->wait_wakeup,
-					     &cluster_lock->wait_mutex,
-					     WAIT_TIME * 2);
+			sem_wait(&cluster_lock->wait_wakeup);
 		} else {
 			sd_debug("failed to call zoo_exists %s", zerror(rc));
 			if (rc != ZNONODE)
@@ -1354,7 +1360,7 @@ static void zk_unlock(uint64_t lock_id)
 static int zk_init(const char *option)
 {
 	char *hosts, *to, *p;
-	int ret;
+	int ret, interval, retry = 0, max_retry;
 
 	if (!option) {
 		sd_err("You must specify zookeeper servers.");
@@ -1374,10 +1380,22 @@ static int zk_init(const char *option)
 		 ZOO_MINOR_VERSION, ZOO_PATCH_VERSION, hosts, zk_timeout);
 	zhandle = zookeeper_init(hosts, zk_watcher, zk_timeout, NULL, NULL, 0);
 	if (!zhandle) {
-		sd_err("failed to connect to zk server %s", option);
+		sd_err("failed to initialize zk server %s", option);
 		return -1;
 	}
 
+	/* the simplest way to wait and check zk connection */
+	interval = 100;
+	max_retry = zk_timeout / interval;
+	while (zoo_state(zhandle) != ZOO_CONNECTED_STATE) {
+		usleep(interval * 1000);
+		if (++retry >= max_retry) {
+			sd_err("failed to connect to zk server %s "
+					"after %d retries", option, retry);
+			return -1;
+		}
+	}
+
 	uatomic_set_false(&stop);
 	uatomic_set_false(&is_master);
 	if (zk_queue_init() != ZOK)
