@@ -697,14 +697,29 @@ main_fn void put_request(struct request *req)
 		eventfd_xwrite(req->local_req_efd, 1);
 	else {
 		if (ci->conn.dead) {
-			clear_client_info(ci);
+			/*
+			 * free_request should be called prior to
+			 * clear_client_info because refcnt of ci will
+			 * be decreased in free_request. Otherwise, ci
+			 * cannot be freed in clear_client_info.
+			 */
 			free_request(req);
+			clear_client_info(ci);
 		} else {
 			list_add_tail(&req->request_list, &ci->done_reqs);
 
 			if (ci->tx_req == NULL)
 				/* There is no request being sent. */
-				conn_tx_on(&ci->conn);
+				if (conn_tx_on(&ci->conn)) {
+					sd_err("switch on sending flag failure, "
+						"connection maybe closed");
+					/*
+					 * should not free_request(req) here
+					 * because it is already in done list
+					 * clear_client_info will free it
+					 */
+					clear_client_info(ci);
+				}
 		}
 	}
 }
@@ -764,7 +779,9 @@ static void rx_main(struct work *work)
 		return;
 	}
 
-	conn_rx_on(&ci->conn);
+	if (conn_rx_on(&ci->conn))
+		sd_err("switch on receiving flag failure, "
+				"connection maybe closed");
 
 	if (is_logging_op(get_sd_op(req->rq.opcode))) {
 		sd_info("req=%p, fd=%d, client=%s:%d, op=%s, data=%s",
@@ -840,7 +857,9 @@ static void tx_main(struct work *work)
 	}
 
 	if (!list_empty(&ci->done_reqs))
-		conn_tx_on(&ci->conn);
+		if (conn_tx_on(&ci->conn))
+			sd_err("switch on sending flag failure, "
+					"connection maybe closed");
 }
 
 static void destroy_client(struct client_info *ci)
@@ -915,12 +934,22 @@ static void client_handler(int fd, int events, void *data)
 
 	sd_debug("%x, %d", events, ci->conn.dead);
 
-	if (events & (EPOLLERR | EPOLLHUP) || ci->conn.dead)
+	if (events & (EPOLLERR | EPOLLHUP))
+		ci->conn.dead = true;
+	/*
+	 * Although dead is true, ci might not be freed immediately
+	 * because of refcnt. Never mind, we will complete it later
+	 * as long as dead is true.
+	 */
+	if (ci->conn.dead)
 		return clear_client_info(ci);
 
 	if (events & EPOLLIN) {
-		if (conn_rx_off(&ci->conn) != 0)
+		if (conn_rx_off(&ci->conn) != 0) {
+			sd_err("switch off receiving flag failure, "
+					"connection maybe closed");
 			return;
+		}
 
 		/*
 		 * Increment refcnt so that the client_info isn't freed while
@@ -933,8 +962,11 @@ static void client_handler(int fd, int events, void *data)
 	}
 
 	if (events & EPOLLOUT) {
-		if (conn_tx_off(&ci->conn) != 0)
+		if (conn_tx_off(&ci->conn) != 0) {
+			sd_err("switch off sending flag failure, "
+					"connection maybe closed");
 			return;
+		}
 
 		assert(ci->tx_req == NULL);
 		ci->tx_req = list_first_entry(&ci->done_reqs, struct request,
@@ -1093,7 +1125,10 @@ worker_fn int sheep_exec_req(const struct node_id *nid, struct sd_req *hdr,
 	}
 	ret = rsp->result;
 	if (ret != SD_RES_SUCCESS)
-		sd_err("failed %s", sd_strerror(ret));
+		sd_err("failed %s, remote address: %s, op name: %s",
+				sd_strerror(ret),
+				addr_to_str(nid->addr, nid->port),
+				op_name(get_sd_op(hdr->opcode)));
 
 	sockfd_cache_put(nid, sfd);
 	return ret;
