@@ -9,7 +9,6 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <sys/signalfd.h>
 #include <sys/resource.h>
 #include <malloc.h>
 
@@ -41,27 +40,17 @@ static const char journal_help[] =
 "\tsize=: size of the journal in megabyes\n"
 "\tdir=: path to the location of the journal (default: $STORE)\n"
 "\tskip: if specified, skip the recovery at startup\n"
-"\nExample:\n\t$ sheep -j dir=/journal,size=1024\n"
-"This tries to use /journal as the journal storage of the size 1024M\n";
-
-static const char loglevel_help[] =
-"Available log levels:\n"
-"  #    Level           Description\n"
-"  0    SDOG_EMERG      system has failed and is unusable\n"
-"  1    SDOG_ALERT      action must be taken immediately\n"
-"  2    SDOG_CRIT       critical conditions\n"
-"  3    SDOG_ERR        error conditions\n"
-"  4    SDOG_WARNING    warning conditions\n"
-"  5    SDOG_NOTICE     normal but significant conditions\n"
-"  6    SDOG_INFO       informational notices\n"
-"  7    SDOG_DEBUG      debugging messages\n"
-"\nExample:\n\t$ sheep -l 4 ...\n"
-"This only allows logs with level smaller than SDOG_WARNING to be logged\n";
+"\nExample:\n\t$ sheep -j dir=/journal,size=1G\n"
+"This tries to use /journal as the journal storage of the size 1G\n";
 
 static const char http_help[] =
-"Example:\n\t$ sheep -r localhost:7001 ...\n"
-"This tries to enable sheep as http service backend and use localhost:7001 to\n"
-"communicate with http server. Not fully implemented yet.\n";
+"Available arguments:\n"
+"\thost=: specify a host to communicate with http server (default: localhost)\n"
+"\tport=: specify a port to communicate with http server (default: 8000)\n"
+"\tswift: enable swift API\n"
+"Example:\n\t$ sheep -r host=localhost,port=7001,swift ...\n"
+"This tries to enable Swift API and use localhost:7001 to\n"
+"communicate with http server.\n";
 
 static const char myaddr_help[] =
 "Example:\n\t$ sheep -y 192.168.1.1:7000 ...\n"
@@ -76,11 +65,11 @@ static const char zone_help[] =
 static const char cluster_help[] =
 "Available arguments:\n"
 "\tlocal: use local driver\n"
-"\tcorosync: use corosync driver (default)\n"
+"\tcorosync: use corosync driver\n"
 "\tzookeeper: use zookeeper driver, need extra arguments\n"
 "\n\tzookeeper arguments: address-list,tiemout=value (default as 3000)\n"
 "\nExample:\n\t"
-"$ sheep -c zookeeperr:IP1:PORT1,IP2:PORT2,IP3:PORT3,timeout=1000 ...\n"
+"$ sheep -c zookeeper:IP1:PORT1,IP2:PORT2,IP3:PORT3,timeout=1000 ...\n"
 "This tries to use 3 node zookeeper cluster, which can be reached by\n"
 "IP1:PORT1, IP2:PORT2, IP3:PORT3 to manage membership and broadcast message\n"
 "and set the timeout of node heartbeat as 1000 milliseconds\n";
@@ -91,34 +80,62 @@ static const char cache_help[] =
 "\tdir=: path to the location of the cache (default: $STORE/cache)\n"
 "\tdirectio: use directio mode for cache IO, "
 "if not specified use buffered IO\n"
-"\nExample:\n\t$ sheep -w size=200000,dir=/my_ssd,directio ...\n"
+"\nExample:\n\t$ sheep -w size=200G,dir=/my_ssd,directio ...\n"
 "This tries to use /my_ssd as the cache storage with 200G allocted to the\n"
 "cache in directio mode\n";
 
+static const char log_help[] =
+"Example:\n\t$ sheep -l dir=/var/log/,level=debug,format=server ...\n"
+"Available arguments:\n"
+"\tdir=: path to the location of sheep.log\n"
+"\tlevel=: log level of sheep.log\n"
+"\tformat=: log format type\n"
+"\tdst=: log destination type\n\n"
+"if dir is not specified, use metastore directory\n\n"
+"Available log levels:\n"
+"  Level      Description\n"
+"  emerg      system has failed and is unusable\n"
+"  alert      action must be taken immediately\n"
+"  crit       critical conditions\n"
+"  err        error conditions\n"
+"  warning    warning conditions\n"
+"  notice     normal but significant conditions\n"
+"  info       informational notices\n"
+"  debug      debugging messages\n"
+"default log level is info\n\n"
+"Available log format:\n"
+"  FormatType      Description\n"
+"  default         raw format\n"
+"  server          raw format with timestamp\n"
+"  json            json format\n\n"
+"Available log destination:\n"
+"  DestinationType    Description\n"
+"  default            dedicated file in a directory used by sheep\n"
+"  syslog             syslog of the system\n"
+"  stdout             standard output\n";
+
 static struct sd_option sheep_options[] = {
 	{'b', "bindaddr", true, "specify IP address of interface to listen on",
 	 bind_help},
 	{'c', "cluster", true,
 	 "specify the cluster driver (default: "DEFAULT_CLUSTER_DRIVER")",
 	 cluster_help},
-	{'d', "debug", false, "include debug messages in the log"},
 	{'D', "directio", false, "use direct IO for backend store"},
-	{'f', "foreground", false, "make the program run in the foreground"},
-	{'F', "log-format", true, "specify log format"},
 	{'g', "gateway", false, "make the progam run as a gateway mode"},
 	{'h', "help", false, "display this help and exit"},
-	{'i', "ioaddr", true, "use separate network card to handle IO requests",
-	 ioaddr_help},
+	{'i', "ioaddr", true, "use separate network card to handle IO requests"
+	 " (default: disabled)", ioaddr_help},
 	{'j', "journal", true, "use jouranl file to log all the write "
-	 "operations", journal_help},
-	{'l', "loglevel", true, "specify the level of logging detail "
-	 "(default: 6 [SDOG_INFO])", loglevel_help},
+	 "operations. (default: disabled)", journal_help},
+	{'l', "log", true,
+	 "specify the log level, the log directory and the log format"
+	 "(log level default: 6 [SDOG_INFO])", log_help},
 	{'n', "nosync", false, "drop O_SYNC for write of backend"},
-	{'o', "stdout", false, "log to stdout instead of shared logger"},
 	{'p', "port", true, "specify the TCP port on which to listen "
 	 "(default: 7000)"},
 	{'P', "pidfile", true, "create a pid file"},
-	{'r', "http", true, "enable http service", http_help},
+	{'r', "http", true, "enable http service. (default: disabled)",
+	 http_help},
 	{'u', "upgrade", false, "upgrade to the latest data layout"},
 	{'v', "version", false, "show the version"},
 	{'w', "cache", true, "enable object cache", cache_help},
@@ -163,15 +180,15 @@ out:
 static void sdlog_help(void)
 {
 	printf("Available log levels:\n"
-	       "  #    Level           Description\n"
-	       "  0    SDOG_EMERG      system has failed and is unusable\n"
-	       "  1    SDOG_ALERT      action must be taken immediately\n"
-	       "  2    SDOG_CRIT       critical conditions\n"
-	       "  3    SDOG_ERR        error conditions\n"
-	       "  4    SDOG_WARNING    warning conditions\n"
-	       "  5    SDOG_NOTICE     normal but significant conditions\n"
-	       "  6    SDOG_INFO       informational notices\n"
-	       "  7    SDOG_DEBUG      debugging messages\n");
+	       "  Level      Description\n"
+	       "  emerg      system has failed and is unusable\n"
+	       "  alert      action must be taken immediately\n"
+	       "  crit       critical conditions\n"
+	       "  err        error conditions\n"
+	       "  warning    warning conditions\n"
+	       "  notice     normal but significant conditions\n"
+	       "  info       informational notices\n"
+	       "  debug      debugging messages\n");
 }
 
 static int create_pidfile(const char *filename)
@@ -258,146 +275,154 @@ static void crash_handler(int signo)
 static struct system_info __sys;
 struct system_info *sys = &__sys;
 
-static void parse_arg(char *arg, const char *delim, void (*fn)(char *))
+static int cache_size_parser(const char *s)
 {
-	char *savep, *s;
-
-	s = strtok_r(arg, delim, &savep);
-	do {
-		fn(s);
-	} while ((s = strtok_r(NULL, delim, &savep)));
-}
-
-static void object_cache_size_set(char *s)
-{
-	const char *header = "size=";
-	int len = strlen(header);
-	char *size, *p;
+	const uint64_t max_cache_size = ((uint64_t)UINT32_MAX + 1)*1024*1024;
 	uint64_t cache_size;
-	const uint32_t max_cache_size = UINT32_MAX;
 
-	assert(!strncmp(s, header, len));
+	if (option_parse_size(s, &cache_size) < 0)
+		return -1;
+#define MIN_CACHE_SIZE (10*1024*1024) /* 10M */
+	if (cache_size < MIN_CACHE_SIZE || cache_size > max_cache_size) {
+		sd_err("Invalid cache option '%s': size must be between "
+		       "between %uM and %" PRIu64 "G", s,
+		       MIN_CACHE_SIZE/1024/1024, max_cache_size/1024/1024/1024);
+		return -1;
+	}
 
-	size = s + len;
-	cache_size = strtoull(size, &p, 10);
-	if (size == p || max_cache_size < cache_size)
-		goto err;
-
-	sys->object_cache_size = cache_size;
-	return;
-
-err:
-	sd_err("Invalid object cache option '%s': size must be an integer "
-	       "between 1 and %" PRIu32 " inclusive", s, max_cache_size);
-	exit(1);
+	sys->object_cache_size = cache_size / 1024 / 1024;
+	return 0;
 }
 
-static void object_cache_directio_set(char *s)
+static int cache_directio_parser(const char *s)
 {
-	assert(!strcmp(s, "directio"));
 	sys->object_cache_directio = true;
+	return 0;
 }
 
 static char ocpath[PATH_MAX];
-static void object_cache_dir_set(char *s)
-{
-	char *p = s;
 
-	p = p + strlen("dir=");
-	snprintf(ocpath, sizeof(ocpath), "%s", p);
+static int cache_dir_parser(const char *s)
+{
+	snprintf(ocpath, sizeof(ocpath), "%s", s);
+	return 0;
 }
 
-static void _object_cache_set(char *s)
+static struct option_parser cache_parsers[] = {
+	{ "size=", cache_size_parser },
+	{ "directio", cache_directio_parser },
+	{ "dir=", cache_dir_parser },
+	{ NULL, NULL },
+};
+
+static int log_level = SDOG_INFO;
+
+static int log_level_parser(const char *s)
 {
-	int i;
+	int level = loglevel_str2num(s);
 
-	struct object_cache_arg {
-		const char *name;
-		void (*set)(char *);
-	};
-
-	struct object_cache_arg object_cache_args[] = {
-		{ "size=", object_cache_size_set },
-		{ "directio", object_cache_directio_set },
-		{ "dir=", object_cache_dir_set },
-		{ NULL, NULL },
-	};
-
-	for (i = 0; object_cache_args[i].name; i++) {
-		const char *n = object_cache_args[i].name;
-
-		if (!strncmp(s, n, strlen(n))) {
-			object_cache_args[i].set(s);
-			return;
-		}
+	if (level < 0) {
+		sd_err("Invalid log level '%s'", s);
+		sdlog_help();
+		return -1;
 	}
 
-	sd_err("invalid object cache arg: %s", s);
-	exit(1);
+	log_level = level;
+	return 0;
 }
 
-static void object_cache_set(char *arg)
+static char *logdir;
+
+static int log_dir_parser(const char *s)
 {
-	sys->enable_object_cache = true;
-	sys->object_cache_size = 0;
-
-	parse_arg(arg, ",", _object_cache_set);
-
-	if (sys->object_cache_size == 0) {
-		sd_err("object cache size is not set");
+	logdir = realpath(s, NULL);
+	if (!logdir) {
+		sd_err("%m");
 		exit(1);
 	}
+	return 0;
 }
 
+static const char *log_format = "server";
+
+static int log_format_parser(const char *s)
+{
+	log_format = s;
+	return 0;
+}
+
+static const char *log_dst = "default"; /* default: dedicated file */
+
+static int log_dst_parser(const char *s)
+{
+	log_dst = s;
+	return 0;
+}
+
+static struct option_parser log_parsers[] = {
+	{ "level=", log_level_parser },
+	{ "dir=", log_dir_parser },
+	{ "format=", log_format_parser },
+	{ "dst=", log_dst_parser },
+	{ NULL, NULL },
+};
+
+
+static const char *io_addr, *io_pt;
+static int ionic_host_parser(const char *s)
+{
+	io_addr = s;
+	return 0;
+}
+
+static int ionic_port_parser(const char *s)
+{
+	io_pt = s;
+	return 0;
+}
+
+static struct option_parser ionic_parsers[] = {
+	{ "host=", ionic_host_parser },
+	{ "port=", ionic_port_parser },
+	{ NULL, NULL },
+};
+
 static char jpath[PATH_MAX];
 static bool jskip;
-static ssize_t jsize;
-#define MIN_JOURNAL_SIZE (64) /* 64M */
+static uint64_t jsize;
 
-static void init_journal_arg(char *arg)
+static int journal_dir_parser(const char *s)
 {
-	const char *d = "dir=", *sz = "size=", *sp = "skip";
-	int dl = strlen(d), szl = strlen(sz), spl = strlen(sp);
-
-	if (!strncmp(d, arg, dl)) {
-		arg += dl;
-		snprintf(jpath, sizeof(jpath), "%s", arg);
-	} else if (!strncmp(sz, arg, szl)) {
-		arg += szl;
-		jsize = strtoll(arg, NULL, 10);
-		if (jsize < MIN_JOURNAL_SIZE || jsize == LLONG_MAX) {
-			sd_err("invalid size %s, must be bigger than %u(M)",
-			       arg,
-				MIN_JOURNAL_SIZE);
-			exit(1);
-		}
-	} else if (!strncmp(sp, arg, spl)) {
-		jskip = true;
-	} else {
-		sd_err("invalid paramters %s", arg);
-		exit(1);
-	}
+	snprintf(jpath, sizeof(jpath), "%s", s);
+	return 0;
 }
 
-static char *io_addr, *io_pt;
-static void init_io_arg(char *arg)
+static int journal_size_parser(const char *s)
 {
-	const char *host = "host=", *port = "port=";
-	int hl = strlen(host), pl = strlen(port);
-
-	if (!strncmp(host, arg, hl)) {
-		arg += hl;
-		io_addr = arg;
-	} else if (!strncmp(port, arg, pl)) {
-		arg += hl;
-		io_pt = arg;
-	} else {
-		sd_err("invalid paramters %s. Use '-i host=a.b.c.d,port=xxx'",
-		       arg);
-		exit(1);
+	if (option_parse_size(s, &jsize) < 0)
+		return -1;
+#define MIN_JOURNAL_SIZE (64*1024*1024) /* 64M */
+	if (jsize < MIN_JOURNAL_SIZE) {
+		sd_err("invalid size %s, must be bigger than %u(M)",
+		       s, MIN_JOURNAL_SIZE/1024/1024);
+		return -1;
 	}
+	return 0;
 }
 
+static int journal_skip_parser(const char *s)
+{
+	jskip = true;
+	return 0;
+}
+
+static struct option_parser journal_parsers[] = {
+	{ "dir=", journal_dir_parser },
+	{ "size=", journal_size_parser },
+	{ "skip", journal_skip_parser },
+	{ NULL, NULL },
+};
+
 static size_t get_nr_nodes(void)
 {
 	struct vnode_info *vinfo;
@@ -416,12 +441,14 @@ static int create_work_queues(void)
 	if (init_work_queue(get_nr_nodes))
 		return -1;
 
+	sys->net_wqueue = create_work_queue("net", WQ_UNLIMITED);
 	sys->gateway_wqueue = create_work_queue("gway", WQ_UNLIMITED);
 	sys->io_wqueue = create_work_queue("io", WQ_UNLIMITED);
 	sys->recovery_wqueue = create_ordered_work_queue("rw");
 	sys->deletion_wqueue = create_ordered_work_queue("deletion");
 	sys->block_wqueue = create_ordered_work_queue("block");
 	sys->md_wqueue = create_ordered_work_queue("md");
+	sys->areq_wqueue = create_work_queue("async_req", WQ_UNLIMITED);
 	if (sys->enable_object_cache) {
 		sys->oc_reclaim_wqueue =
 			create_ordered_work_queue("oc_reclaim");
@@ -430,7 +457,8 @@ static int create_work_queues(void)
 			return -1;
 	}
 	if (!sys->gateway_wqueue || !sys->io_wqueue || !sys->recovery_wqueue ||
-	    !sys->deletion_wqueue || !sys->block_wqueue || !sys->md_wqueue)
+	    !sys->deletion_wqueue || !sys->block_wqueue || !sys->md_wqueue ||
+	    !sys->areq_wqueue)
 			return -1;
 
 	return 0;
@@ -457,8 +485,8 @@ static void check_host_env(void)
 	 * dangerous to run Sheepdog cluster.
 	 */
 	else if (r.rlim_cur == 1024)
-		sd_err("WARN: Allowed open files 1024 too small, suggested %u",
-		       SD_RLIM_NOFILE);
+		sd_warn("Allowed open files 1024 too small, suggested %u",
+			SD_RLIM_NOFILE);
 	else if (r.rlim_cur < SD_RLIM_NOFILE)
 		sd_info("Allowed open files %lu, suggested %u", r.rlim_cur,
 			SD_RLIM_NOFILE);
@@ -570,23 +598,36 @@ end:
 	return status;
 }
 
+static void sighup_handler(int signum)
+{
+	if (unlikely(logger_pid == -1))
+		return;
+
+	/* forward SIGHUP for log rotating */
+	kill(logger_pid, SIGHUP);
+}
+
 int main(int argc, char **argv)
 {
 	int ch, longindex, ret, port = SD_LISTEN_PORT, io_port = SD_LISTEN_PORT;
-	int log_level = SDOG_INFO, nr_vnodes = SD_DEFAULT_VNODES;
+	int nr_vnodes = SD_DEFAULT_VNODES;
 	const char *dirp = DEFAULT_OBJECT_DIR, *short_options;
-	char *dir, *p, *pid_file = NULL, *bindaddr = NULL, path[PATH_MAX],
+	char *dir, *p, *pid_file = NULL, *bindaddr = NULL, log_path[PATH_MAX],
 	     *argp = NULL;
-	bool is_daemon = true, to_stdout = false, explicit_addr = false;
+	bool explicit_addr = false;
 	int64_t zone = -1;
 	struct cluster_driver *cdrv;
 	struct option *long_options;
-	const char *log_format = "server", *http_address = NULL;
+	const char *http_options = NULL;
 	static struct logger_user_info sheep_info;
+	struct stat logdir_st;
+	enum log_dst_type log_dst_type;
 
 	install_crash_handler(crash_handler);
 	signal(SIGPIPE, SIG_IGN);
 
+	install_sighandler(SIGHUP, sighup_handler, false);
+
 	long_options = build_long_options(sheep_options);
 	short_options = build_short_options(sheep_options);
 	while ((ch = getopt_long(argc, argv, short_options, long_options,
@@ -604,19 +645,11 @@ int main(int argc, char **argv)
 			pid_file = optarg;
 			break;
 		case 'r':
-			http_address = optarg;
-			break;
-		case 'f':
-			is_daemon = false;
+			http_options = optarg;
 			break;
 		case 'l':
-			log_level = strtol(optarg, &p, 10);
-			if (optarg == p || log_level < SDOG_EMERG ||
-			    SDOG_DEBUG < log_level || *p != '\0') {
-				sd_err("Invalid log level '%s'", optarg);
-				sdlog_help();
+			if (option_parse(optarg, ",", log_parsers) < 0)
 				exit(1);
-			}
 			break;
 		case 'n':
 			sys->nosync = true;
@@ -628,10 +661,6 @@ int main(int argc, char **argv)
 			}
 			explicit_addr = true;
 			break;
-		case 'd':
-			/* removed soon. use loglevel instead */
-			log_level = SDOG_DEBUG;
-			break;
 		case 'D':
 			sys->backend_dio = true;
 			break;
@@ -639,9 +668,6 @@ int main(int argc, char **argv)
 			/* same as '-v 0' */
 			nr_vnodes = 0;
 			break;
-		case 'o':
-			to_stdout = true;
-			break;
 		case 'z':
 			zone = strtol(optarg, &p, 10);
 			if (optarg == p || zone < 0 || UINT32_MAX < zone
@@ -671,10 +697,21 @@ int main(int argc, char **argv)
 			sys->cdrv_option = get_cdrv_option(sys->cdrv, optarg);
 			break;
 		case 'w':
-			object_cache_set(optarg);
+			sys->enable_object_cache = true;
+			sys->object_cache_size = 0;
+
+			if (option_parse(optarg, ",", cache_parsers) < 0)
+				exit(1);
+
+			if (sys->object_cache_size == 0) {
+				sd_err("object cache size is not set");
+				exit(1);
+			}
 			break;
 		case 'i':
-			parse_arg(optarg, ",", init_io_arg);
+			if (option_parse(optarg, ",", ionic_parsers) < 0)
+				exit(1);
+
 			if (!str_to_addr(io_addr, sys->this_node.nid.io_addr)) {
 				sd_err("Bad addr: '%s'", io_addr);
 				exit(1);
@@ -689,7 +726,8 @@ int main(int argc, char **argv)
 			break;
 		case 'j':
 			uatomic_set_true(&sys->use_journal);
-			parse_arg(optarg, ",", init_journal_arg);
+			if (option_parse(optarg, ",", journal_parsers) < 0)
+				exit(1);
 			if (!jsize) {
 				sd_err("you must specify size for journal");
 				exit(1);
@@ -708,9 +746,6 @@ int main(int argc, char **argv)
 				PACKAGE_VERSION);
 			exit(0);
 			break;
-		case 'F':
-			log_format = optarg;
-			break;
 		default:
 			usage(1);
 			break;
@@ -740,14 +775,50 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
-	snprintf(path, sizeof(path), "%s/" LOG_FILE_NAME, dir);
+	if (!strcmp(log_dst, "default"))
+		log_dst_type = LOG_DST_DEFAULT;
+	else if (!strcmp(log_dst, "stdout"))
+		log_dst_type = LOG_DST_STDOUT;
+	else if (!strcmp(log_dst, "syslog"))
+		log_dst_type = LOG_DST_SYSLOG;
+	else {
+		sd_err("invalid type of log destination: %s", log_dst);
+		exit(1);
+	}
+
+	if (logdir) {
+		if (log_dst_type != LOG_DST_DEFAULT) {
+			sd_err("logdir (%s) is specified but logging"
+			       " destination is %s", logdir,
+			       log_dst_type == LOG_DST_STDOUT
+			       ? "stdout" : "syslog");
+			exit(1);
+		}
+
+		memset(&logdir_st, 0, sizeof(logdir_st));
+		ret = stat(logdir, &logdir_st);
+		if (ret < 0) {
+			sd_err("stat() failed on %s, %m", logdir);
+			exit(1);
+		}
+
+		if (!S_ISDIR(logdir_st.st_mode)) {
+			sd_err("log dir: %s is not a directory", logdir);
+			exit(1);
+		}
+	}
+
+	snprintf(log_path, sizeof(log_path), "%s/" LOG_FILE_NAME,
+		 logdir ?: dir);
+
+	free(logdir);
 
 	srandom(port);
 
-	if (lock_and_daemon(is_daemon, dir))
+	if (lock_and_daemon(log_dst_type != LOG_DST_STDOUT, dir))
 		exit(1);
 
-	ret = log_init(program_name, to_stdout, log_level, path);
+	ret = log_init(program_name, log_dst_type, log_level, log_path);
 	if (ret)
 		exit(1);
 
@@ -775,7 +846,7 @@ int main(int argc, char **argv)
 	if (ret)
 		exit(1);
 
-	local_req_init();
+	local_request_init();
 
 	ret = init_signal();
 	if (ret)
@@ -797,12 +868,14 @@ int main(int argc, char **argv)
 		if (!strlen(jpath))
 			/* internal journal */
 			memcpy(jpath, dir, strlen(dir));
-		sd_debug("%s, %zd, %d", jpath, jsize, jskip);
+		sd_debug("%s, %"PRIu64", %d", jpath, jsize, jskip);
 		ret = journal_file_init(jpath, jsize, jskip);
 		if (ret)
 			exit(1);
 	}
 
+	init_fec();
+
 	/*
 	 * After this function, we are multi-threaded.
 	 *
@@ -835,7 +908,7 @@ int main(int argc, char **argv)
 	if (ret)
 		exit(1);
 
-	if (http_address && http_init(http_address) != 0)
+	if (http_options && http_init(http_options) != 0)
 		exit(1);
 
 	if (pid_file && (create_pidfile(pid_file) != 0)) {
