@@ -43,7 +43,7 @@ int shadow_file_read(const char *path, char *buf, size_t size, off_t offset)
 	return len;
 }
 
-size_t shadow_file_write(const char *path, char *buf, size_t size)
+size_t shadow_file_write(const char *path, const char *buf, size_t size)
 {
 	char p[PATH_MAX];
 	int fd;
@@ -92,6 +92,18 @@ int shadow_dir_create(const char *path)
 	return 0;
 }
 
+int shadow_dir_delete(const char *path)
+{
+	char p[PATH_MAX];
+
+	snprintf(p, sizeof(p), "%s%s", sheepfs_shadow, path);
+	if (rmdir(p) < 0) {
+		sheepfs_pr("%m\n");
+		return -1;
+	}
+	return 0;
+}
+
 int shadow_file_setxattr(const char *path, const char *name,
 			 const void *value, size_t size)
 {
@@ -145,3 +157,17 @@ bool shadow_file_exsit(const char *path)
 
 	return true;
 }
+
+bool shadow_file_stat(const char *path, struct stat *st)
+{
+	char p[PATH_MAX];
+
+	snprintf(p, sizeof(p), "%s%s", sheepfs_shadow, path);
+	if (stat(p, st) < 0) {
+		if (errno != ENOENT)
+			sheepfs_pr("%m\n");
+		return false;
+	}
+
+	return true;
+}
